from Products.ERP5Type.tests.SecurityTestCase import SecurityTestCase
import time
from StringIO import StringIO
import zipfile
import httplib
import urlparse


class LocalStringIO(StringIO):
  __allow_access_to_unprotected_subobjects__=1
  def __init__(self, filename, *args, **kw):
    self.filename = filename
    StringIO.__init__(self, *args, **kw)


class TestOfficeJSScenario(SecurityTestCase):
  def getTitle(self):
    return "Test OfficeJS Scenario"

  def getBusinessTemplateList(self):
    """
    Tuple of Business Templates we need to install
    """
    return ('officejs_meta',)

  def createNewUser(self,
                 first_name="John_test",
                 last_name="Smith_test",
                 email="john.smith@example.com"):
    """
    Add new member using script for the website sign-in
    """
    user = self.portal.person_module.newContent(
      portal_type="Person",
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
      career_role_list=["author"]
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", role="author")
    assignment.open()
    return user

  def createNewAdmin(self,
                 first_name="Brandon_test",
                 last_name="Cougar_test",
                 email="brandon.cougar@example.com"):
    """
    Add new member using script for the website sign-in
    """
    user = self.portal.person_module.newContent(
      portal_type="Person",
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
      career_role_list=["administration"]
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", group="my_group")
    assignment.open()
    return user

  def test_01_user_can_contribute(self):
    timestamp = "%s" % time.time()
    ###################################
    ### Manager: Create user
    person_user = self.createNewUser()
    person_admin = self.createNewAdmin()
    self.tic()

    ###################################
    ### User: submit application
    self.logout()
    self.login(person_user.getUserId())

    import_file = LocalStringIO('your.zip')

    my_zip = zipfile.ZipFile(import_file, 'w')
    my_zip.writestr('index.html', 'My super content')
    for i in range(50):
      my_zip.writestr('gadget_n_' + str(i) + '.html', 'My super gadget ' + str(i))
    my_zip.close()

    version = 'My Bar version %s' % timestamp
    self.portal.ERP5Site_createNewSoftwarePublication(
      import_file,
      'software/application',
      'My Foo App',
      version
    )

    ###################################
    ### Alarm: extract zip file
    self.logout()
    self.login()
    self.portal.portal_alarms.officejs_process_submit_software_publication\
                             .activeSense()
    self.portal.portal_alarms.officejs_process_submit_software_publication\
                             .activeSense()
    self.tic()
    self.portal.portal_alarms.officejs_process_submit_software_publication\
                             .activeSense()
    self.tic()
    web_section = self.portal.portal_catalog.getResultValue(
      portal_type='Static Web Section',
      title=version
    )

    ###################################
    ### Anonymous: access the redirect application
    self.logout()
    api_scheme, api_netloc, api_path, api_query, \
      api_fragment = urlparse.urlsplit(self.portal.absolute_url())

    connection = httplib.HTTPConnection(api_netloc)

    connection.request(
      method='GET',
      url='%s/' % web_section.getParentValue().absolute_url()
    )
    response = connection.getresponse()

    page_content = response.read()
    self.assertTrue(
      'document.location.replace("development/" + document.location.hash)' in page_content,
      page_content
    )
    self.assertTrue(
      'manifest="WebSection_renderOfficeJSRedirectAppCache"' in page_content,
      page_content
    )
    self.assertEquals(response.status, 200)
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=0, public, must-revalidate'
    )
    self.assertTrue(
      'text/html;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Content-Security-Policy'),
      "default-src 'self'; script-src 'unsafe-inline';"
    )
    self.assertEquals(
      response.getheader('X-Frame-Options'),
      'SAMEORIGIN'
    )
    self.assertEquals(
      response.getheader('Vary').replace(' ', ''),
      'Accept-Language,Cookie,Authorization,Accept-Encoding'
    )


    ###################################
    ### Anonymous: access the redirect application appcache
    connection.request(
      method='GET',
      url='%s/WebSection_renderOfficeJSRedirectAppCache' % web_section.getParentValue().absolute_url()
    )
    response = connection.getresponse()

    page_content = response.read()
    self.assertEquals("""CACHE MANIFEST
# development + hash""",
      page_content
    )
    self.assertEquals(response.status, 200)

    self.assertTrue(
      'text/cache-manifest;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=0, public, must-revalidate'
    )
    self.assertEquals(
      response.getheader('Vary'),
      None
    )

    ###################################
    ### Anonymous: access the application version
    connection.request(
      method='GET',
      url='%s/' % web_section.absolute_url()
    )
    response = connection.getresponse()

    self.assertEquals(response.read(), 'My super content')
    self.assertEquals(response.status, 200)
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=31536000, stale-while-revalidate=31536000, stale-if-error=31536000, public'
    )
    self.assertTrue(
      'text/html;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Content-Security-Policy'),
      None
    )
    self.assertEquals(
      response.getheader('X-Frame-Options'),
      None
    )
    self.assertEquals(response.getheader('Vary'), None)

    ###################################
    ### User: open the publication to be accepted
    self.logout()
    self.login(person_user.getUserId())

    software_publication = self.portal.portal_catalog.getResultValue(
      portal_type='Software Publication',
      reference='SP-%s' % web_section.getId()
    )
    software_publication.open()
    self.tic()

    ###################################
    ### Anonymous: access the redirect application
    self.logout()
    api_scheme, api_netloc, api_path, api_query, \
      api_fragment = urlparse.urlsplit(self.portal.absolute_url())

    connection = httplib.HTTPConnection(api_netloc)

    connection.request(
      method='GET',
      url='%s/' % web_section.getParentValue().absolute_url()
    )
    response = connection.getresponse()

    page_content = response.read()
    self.assertTrue(
      'document.location.replace("development/" + document.location.hash)' in page_content,
      page_content
    )
    self.assertTrue(
      'manifest="WebSection_renderOfficeJSRedirectAppCache"' in page_content,
      page_content
    )
    self.assertEquals(response.status, 200)
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=0, public, must-revalidate'
    )
    self.assertTrue(
      'text/html;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Content-Security-Policy'),
      "default-src 'self'; script-src 'unsafe-inline';"
    )
    self.assertEquals(
      response.getheader('X-Frame-Options'),
      'SAMEORIGIN'
    )
    self.assertEquals(
      response.getheader('Vary').replace(' ', ''),
      'Accept-Language,Cookie,Authorization,Accept-Encoding'
    )

    ###################################
    ### Site admin: accept the software publication
    self.logout()
    self.login(person_admin.getUserId())

    software_publication.accept()
    self.tic()

    ###################################
    ### Anonymous: access the redirect application
    self.logout()
    connection.request(
      method='GET',
      url='%s/' % web_section.getParentValue().absolute_url()
    )
    response = connection.getresponse()

    page_content = response.read()
    self.assertTrue(
      'document.location.replace("%s/" + document.location.hash)' % web_section.getId() in page_content,
      page_content
    )
    self.assertTrue(
      'manifest="WebSection_renderOfficeJSRedirectAppCache"' in page_content,
      page_content
    )
    self.assertEquals(response.status, 200)
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=0, public, must-revalidate'
    )
    self.assertTrue(
      'text/html;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Content-Security-Policy'),
      "default-src 'self'; script-src 'unsafe-inline';"
    )
    self.assertEquals(
      response.getheader('X-Frame-Options'),
      'SAMEORIGIN'
    )
    self.assertEquals(
      response.getheader('Vary').replace(' ', ''),
      'Accept-Language,Cookie,Authorization,Accept-Encoding'
    )

    ###################################
    ### Anonymous: access the redirect application appcache
    connection.request(
      method='GET',
      url='%s/WebSection_renderOfficeJSRedirectAppCache' % web_section.getParentValue().absolute_url()
    )
    response = connection.getresponse()

    page_content = response.read()
    self.assertEquals("""CACHE MANIFEST
# %s + hash""" % web_section.getId(),
      page_content
    )
    self.assertEquals(response.status, 200)

    self.assertTrue(
      'text/cache-manifest;' in response.getheader('Content-Type'),
      response.getheader('Content-Type')
    )
    self.assertEquals(
      response.getheader('Cache-Control'),
      'max-age=0, public, must-revalidate'
    )
    self.assertEquals(
      response.getheader('Vary'),
      None
    )
