from Products.ERP5Type.tests.SecurityTestCase import SecurityTestCase
import time
from StringIO import StringIO
import zipfile
import json

class LocalStringIO(StringIO):
  __allow_access_to_unprotected_subobjects__=1
  def __init__(self, filename, *args, **kw):
    self.filename = filename
    StringIO.__init__(self, *args, **kw)


class TestOfficeJSScenario(SecurityTestCase):
  def getTitle(self):
    return "Test OfficeJS Scenario Appstore"

  def getBusinessTemplateList(self):
    """
    Tuple of Business Templates we need to install
    """
    return ('officejs_meta', 'officejs_credential')

  def createNewUser(self,
                 first_name="John_test",
                 last_name="Smith_test",
                 email="john.smith@example.com"):
    """
    Add new member using script for the website sign-in
    """
    user = self.portal.person_module.newContent(
      portal_type="Person",
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
      career_role_list=["author"]
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", role="author")
    assignment.open()
    return user

  def createNewAdmin(self,
                 first_name="Brandon_test",
                 last_name="Cougar_test",
                 email="brandon.cougar@example.com"):
    """
    Add new member using script for the website sign-in
    """
    user = self.portal.person_module.newContent(
      portal_type="Person",
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
      career_role_list=["administration"]
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", group="my_group")
    assignment.open()
    return user

  def createApplicationAndPublish(self, title):
    person_user = self.createNewUser()
    person_admin = self.createNewAdmin()
    self.tic()
    self.login(person_user.getUserId())
    import_file = LocalStringIO('app.zip')
    my_zip = zipfile.ZipFile(import_file, 'w')
    my_zip.writestr('index.html', '<html><head><link rel="manifest" href="app_manifest.json"></head></html>')
    my_zip.writestr('app_manifest.json', '{"short_name": "%s","name": "%s","description": "application test", "icons": [{"src": "logo.png","sizes": "any","type": "image/png"}],"start_url": "../","display": "standalone"}' % (title, title))
    my_zip.writestr('logo.png', 'skapoutch')
    my_zip.close()
    self.portal.SoftwareProductModule_createNewApplication(
      import_file,
      title
    )
    self.logout()
    self.login()

    self.tic()
    self.portal.portal_alarms.officejs_process_submit_software_publication\
                             .activeSense()
    self.tic()
    software_product = self.portal.portal_catalog.getResultValue(
      portal_type='Software Product',
      title=title
    )
    web_site = software_product.SoftwareProduct_getRelatedWebSite()
    web_section = web_site.objectValues()[0]
    self.logout()
    self.login(person_user.getUserId())

    software_publication = self.portal.portal_catalog.getResultValue(
      portal_type='Software Publication',
      reference='SP-%s' % web_section.getId()
    )
    software_publication.open()
    self.tic()
    self.logout()
    self.login(person_admin.getUserId())

    software_publication.accept()
    self.tic()
    self.logout()
    self.login()
    self.tic()
    self.portal.portal_alarms.officejs_process_update_appstore.activeSense()
    self.tic()

  def getApplicationWithTitle(self, title):
    application_list = json.loads(self.portal.document_module['store_officejs_data_application_sample_json'].getData())
    for app in application_list:
      if app['application_title'] == title:
        return app
    return None

  def checkApplicationWithTitle(self, title):
    app = self.getApplicationWithTitle(title)
    self.assertTrue(app is not None)
    self.assertTrue('logo.png' in app['application_image_url'])
    self.assertEquals(app['application_description'], 'application test')
    self.assertEquals(app['application_image_type'], 'image')

  def test_appstore_add_an_application(self):
    title = 'My Application %s' % time.time()
    self.assertEquals(self.getApplicationWithTitle(title), None)
    self.createApplicationAndPublish(title)
    self.checkApplicationWithTitle(title)
    return title

  def test_appstore_add_multiple_application(self):
    t_list = []
    for _ in range(4):
      t_list.append(self.test_appstore_add_an_application())
    for t in t_list:
      self.checkApplicationWithTitle(t)
